function optimal_pole = arg_max(ze,NN,real_part,imaginary_part)

%ARG_MAX Compute an optimal dominating pole by minimizing the squared error given in section 7.2
%
%   OPTIMAL_POLE = ARG_MAX(ze,NN,real_part,imaginary_part)
% 
%   This OPTIMAL POLE is derived, based on the model
%   A(q) y(t) = B(q) u(t-nk) + e(t)
%  
%   ZE: the output-input data ZE=[ye ue] for estimation process,
%       with ye and ue as column vectors.
%
%   NN: NN = [na nb nk], the orders and true delays of the model.
%       if na=0 the model will correspond to FIR model.
%
%   REAL_PART: real_part = [started_point:step_size:ended_point] 
%       where abs(started_point) < 1 and abs(ended_point) < 1
%
%   IMAGINARY_PART: imaginary_part = [started_point:step_size:ended_point] 
%       where abs(started_point) < 1 and abs(ended_point) < 1

%   Piya Kovintavewat
%   Revision:1       Date: 1998/10/15 

if nargin<4, imaginary_part=0; end;

NA = NN(1);    NB=NN(2);  
NK = NN(3);    if NK<0, NK=0; end;
maxlength =length(ze(:,1));
if length(real_part)>1, step_size_r=real_part(2)-real_part(1); else, step_size_r=0; end;
if length(imaginary_part)>1, step_size_c=imaginary_part(2)-imaginary_part(1); else, step_size_c=0; end; 

nmax  = max([NA+1, NB+NK+1]);   ii=nmax:maxlength;   N=length(ii); 

a_max = [];
for i=1:length(real_part)
    for j=1:length(imaginary_part)
        poles = [];     pole = real_part(i)+imaginary_part(j); 
        if abs(pole) < 0.99
           aPHI=[]; aPHIV=[]; bPHI=[]; bPHIV=[]; PHI=[]; PHIV=[];
           if abs(imag(pole))>0   %---KAUTZ model
              poles=[];      poles(1:NB/2)=pole; 
              [bPHI,bPHIV] = phi_gobfilt_arx(ze,ze,poles,0);
              if NA~=0   %---ARX structure     
                 poles=[];      poles(1:NA/2)=pole; 
                 [aPHI,aPHIV] = phi_gobfilt_arx(ze,ze,poles,1);
                 PHI          = [aPHI(ii,:),bPHI(ii-NK,:)];   
              else       %---FIR structure   
                 PHI          = bPHI(ii-NK,:);   
              end;
           else   %---LAGUERRE model
              poles=[];      poles(1:NB)=pole; 
              [bPHI,bPHIV] = phi_gobfilt_arx(ze,ze,poles,0);  
              if NA~=0   %---ARX structure  
                 poles=[];      poles(1:NA)=pole; 
                 [aPHI,aPHIV] = phi_gobfilt_arx(ze,ze,poles,1);
                 PHI          = [aPHI(ii,:),bPHI(ii-NK,:)];
              else       %---FIR structure 
                 PHI          = bPHI(ii-NK,:);
              end;
           end;
           Town  = (PHI'*PHI)/N;  
           gamma = (ze(ii,1)'*PHI)/N;   
           a_max(i,j) = gamma*inv(Town)*gamma';  
        else
           a_max(i,j) = 0;
        end;
    end;
end 
   
[yimax,irow] = max(a_max);           
[yjmax,jcol] = max(max(a_max));      
optimal_pole = (real_part(1)+(irow(jcol)-1)*step_size_r) + (imaginary_part(1)+(jcol-1)*step_size_c);
               
