function optimal_pole = arg_max_2stages(ze,NN,real_part,imaginary_part)

%ARG_MAX_2STAGES compute an optimal pole of a TWO-STAGE ECHO CANCELER, where the first stage
%   is a conventional FIR model of NB parameters and the second stage is an orthonormal model
%   with FIR structure of NA parameters. It is done by minimizing the squared error given in section 7.2.
%
%   OPTIMAL_POLE = ARG_MAX_2STAGES(ze,NN,real_part,imaginary_part)
%
%   This OPTIMAL POLE is derived from a TWO-STAGES ECHO CANCELLER.
%   The first stage of NB parameters is given by y_fir(t)   = B(q) u(t-nk), while 
%   the second stage of NA parameters is given by y_ortho(t) = A(g) u(t-nk-nb). 
%   Consequently, the estimated output is y(t) = y_fir(t) + y_ortho(t).   
%
%   ZE: the output-input data ZE=[ye ue] for estimation process,
%       with ye and ue as column vectors.
%
%   NN: NN = [na nb nk], the orders of each stages(na,nb) and true delays(nk) of the model.
%       if na=0 the model will correspond to FIR model.
%
%   REAL_PART: real_part = [started_point:step_size:ended_point] 
%       where abs(started_point) < 1 and abs(ended_point) < 1
%
%   IMAGINARY_PART: imaginary_part = [started_point:step_size:ended_point] 
%       where abs(started_point) < 1 and abs(ended_point) < 1
%

%   Piya Kovintavewat
%   Revision:1       Date: 1998/10/15 

NA = NN(1);    NB=NN(2);  
NK = NN(3);    if NK<0, NK=0; end;
maxlength =length(ze(:,1));
if length(real_part)>1, step_size_r=real_part(2)-real_part(1); else, step_size_r=0; end;
if length(imaginary_part)>1, step_size_c=imaginary_part(2)-imaginary_part(1); else, step_size_c=0; end; 

%---Calculate the regression matrix of the first part which is FIR of order NB 
poles  = [];  poles   = zeros(1,NB);     
firPHI = [];  firPHIV = []; [firPHI,firPHIV] = phi_gobfilt_arx(ze,zv,poles,0);

nmax  = max([1, NA+NB+NK+1]);   ii=nmax:maxlength;   N=length(ii); 
a_max = [];
for i=1:length(real_part)
    for j=1:length(imaginary_part)
        poles = [];      pole = real_part(i)+imaginary_part(j); 
        if abs(pole) < 0.99
           if abs(imag(pole))>0   %---Kautz model
              poles = ones(1,NA/2)*pole; 
           else   %---Laguerre model
              poles = ones(1,NA)*pole;
           end;
           bPHI  = [];  bPHIV=[];  [bPHI,bPHIV] = phi_gobfilt_arx(ze,ze,poles,0);   
           PHI   = [];  PHI   = [firPHI(ii-NK,:), bPHI(ii-NK-NB,:)];
           Town  = (PHI'*PHI)/N;  
           gamma = (ze(ii,1)'*PHI)/N;   
           a_max(i,j) = gamma*inv(Town)*gamma';        
        else
           a_max(i,j) = 0;
        end;
    end;
end; 

[yimax,irow] = max(a_max);           
[yjmax,jcol] = max(max(a_max));      
optimal_pole = (real_part(1)+(irow(jcol)-1)*step_size_r)+(imaginary_part(1)+(jcol-1)*step_size_c);
   
   