function fit = fit_2stages(ze,zv,NN,pole)

%FIT_2STAGES compute the FIT of error of a TWO-STAGES ECHO CANCELER MODEL
%
%   FIT = FIT_2STAGES(ze,zv,NN,pole) 
% 
%   This FIT is calculated from a TWO-STAGES ECHO CANCELER model, where the first stage is 
%   a conventional FIR model of NB parameters and the second stage is an orthonormal model
%   with FIR structure of NA parameters.
%
%   ZE: the output-input data ZE=[ye ue] for estimation process,
%       with ye and ue as column vectors.
%
%   ZV: the output-input data ZV=[yv uv] having the same size as ZE for validation process,
%       with yv and uv as column vectors.
%
%   NN: NN = [na nb nk], the orders of each stages(na,nb) and true delays(nk) of the model.
%
%   POLE: The optimal pole corresponds to a priori information about the true dynamics,
%       where a real-valued pole corresponds to LAGUERRE model and 
%       a complex-valued pole corresponds to KAUTZ model. 
%       We use the same pole for every model order specified by NA.   
% 

%
%   Piya Kovintavewat
%   Revision:1       Date: 1998/10/15 

NA = NN(1);   NB=NN(2); 
NK = NN(3);   if NK<0, NK=0; end;
vz=1; if size(ze)==size(zv), if norm(ze-zv)<eps, vz=0; end;  end;
maxlength = length(ze(:,1));
nmax  = max([1, NA+NB+NK+1]);   ii=nmax:maxlength;   N=length(ii); 

%----------------------Calculating the first part as FIR of order NB 
poles  = [];  poles   = zeros(1,NB);     
firPHI = [];  firPHIV = []; [firPHI,firPHIV] = phi_gobfilt_arx(ze,zv,poles,0);

poles=[];
if abs(imag(pole))>0, poles = ones(1,NA/2)*pole; else, poles = ones(1,NA)*pole; end;    
bPHI  = [];  bPHIV = [];  [bPHI,bPHIV] = phi_gobfilt_arx(ze,zv,poles,0);   
PHI   = [];  PHI   = [firPHI(ii-NK,:),  bPHI(ii-NK-NB,:)];
PHIV  = [];  PHIV  = [firPHIV(ii-NK,:), bPHIV(ii-NK-NB,:)];
THETA = PHI\ze(ii,1);
if vz==0,  ysim=PHI*THETA; else, ysim=PHIV*THETA; end;
fit = norm(ysim-zv(ii,1))/sqrt(length(ysim));

