function fit = fit_orthonormal(ze,zv,NN,pole)

%FIT_ORTHONORMAL Compute the FIT of error of LS-estimates of LAGUERRE/KAUTZ models
%
%   FIT = FIT_ORTHONORMAL(ze,zv,NN,pole) 
% 
%   This FIT is calculated from the Laguerre/Kautz models implemented with FIR or ARX structure as
%       A(q) y(t) = B(q) u(t-nk) + e(t)
%       by using cross validation method to varify the model.
%  
%   ZE: the output-input data ZE=[ye ue] for estimation process,
%       with ye and ue as column vectors.
%
%   ZV: the output-input data ZV=[yv uv] having the same size as ZE for validation process,
%       with yv and uv as column vectors.
%
%   NN: NN = [na nb nk], the orders and true delays of the model.
%       if na=0 the model will correspond to FIR structure.
%
%   POLE: The dominating pole corresponds to a priori information about the true dynamics,
%       where a real-valued pole corresponds to LAGUERRE model,
%       and a complex-valued pole corresponds to KAUTZ model. 
%       We use the same pole for every model order specified by [na nb].   

%   Piya Kovintavewat
%   Revision:1       Date: 1998/10/15 

NA = NN(1);   NB=NN(2); 
NK = NN(3);   if NK<0, NK=0; end;
vz=1; if size(ze)==size(zv), if norm(ze-zv)<eps, vz=0; end;  end;
maxlength = length(ze(:,1));
nmax  = max([NA+1, NB+NK+1]);   ii=nmax:maxlength;

if abs(imag(pole)) > 0   %---Kautz model
   aPHI =[]; aPHIV=[]; bPHI=[]; bPHIV=[]; PHI=[]; PHIV=[]; 
   poles=[];      poles(1:NB/2)=pole; 
   [bPHI,bPHIV] = phi_gobfilt_arx(ze,zv,poles,0);  
   if NA~=0   %---ARX structure
      poles=[];      poles(1:NA/2)=pole; 
      [aPHI,aPHIV] = phi_gobfilt_arx(ze,zv,poles,1);
      PHI          = [aPHI(ii,:), bPHI(ii-NK,:)]; 
      if vz==1, PHIV=[aPHIV(ii,:),bPHIV(ii-NK,:)]; end;
   else       %---FIR structure
      PHI          = bPHI(ii-NK,:);
      if vz==1, PHIV=bPHIV(ii-NK,:); end;
   end;   
else  %---Laguerre model
   aPHI =[]; aPHIV=[]; bPHI=[]; bPHIV=[]; PHI=[]; PHIV=[]; 
   poles=[];      poles(1:NB)=pole; 
   [bPHI,bPHIV] = phi_gobfilt_arx(ze,zv,poles,0);
   if NA~=0   %---ARX structure
      poles=[];      poles(1:NA)=pole; 
      [aPHI,aPHIV] = phi_gobfilt_arx(ze,zv,poles,1);
      PHI          = [aPHI(ii,:), bPHI(ii-NK,:)]; 
      if vz==1, PHIV=[aPHIV(ii,:),bPHIV(ii-NK,:)]; end;
   else       %---FIR structure
      PHI          = bPHI(ii-NK,:); 
      if vz==1, PHIV=bPHIV(ii-NK,:); end;
   end;
end

THETA    = PHI\ze(ii,1);
if vz==0, ysim=PHI*THETA; else, ysim=PHIV*THETA; end;
fit = norm(ysim-zv(ii,1))/sqrt(length(ysim)); 
