function [TH,PHI,PHIV] = mo_arxstruc(ze,zv,nn);

%MO_ARXSTRUC computes the resgression matrix and estimated parameters of ARX-model.
%   [TH,PHI,PHIV] = MO_ARXSTRUC(z,zv,nn)
%
%   This function is modified from a traditional "ARXSTRUC.M" in order to generate a 
%   regression matrices(PHI and PHIV) and estimated parameter vector(TH) because the 
%   traditional ARXSTRUC.M does not take care about the transient effect from an unknown 
%   initial value.
%
%   TH: returned as the estimated parameters of the ARX model
%   A(q) y(t) = B(q) u(t-nk) + e(t)
%   
%   PHI: contains the regression matrix which is generated by a set of estimation data(ze).
%       Then it is used for estimation process.
% 
%   PHIV: contains the regression matrix which is generated by a set of validation data(zv).
%       Then it is used for validation process.
%
%   ZE: the output-input data ZE=[ye ue] for estimation process,
%       with ye and ue as column vectors.
%
%   ZV: the output-input data ZV=[yv uv] having the same size as ZE for validation process,
%       with yv and uv as column vectors.
%
%   NN: NN = [na nb nk], the orders and true delays of the model.
%       if na=0 the model will correspond to FIR model.

%   Piya Kovintavewat
%   Revision:1       Date: 1998/10/15 

z=ze;
[Ncap,nz] = size(z);    nu = nz-1;
[nm,nl]   = size(nn);   [Ncapv,nzv] = size(zv);

if nz>1, na=nn(:,1);nb=nn(:,2:1+nu);nk=nn(:,2+nu:1+2*nu);
else na=nn(:,1); nb=zeros(nm,1);nk=zeros(nm,1);end

nma=max(na);
nbkm=max(nb+nk)-ones(1,nu);
nkm=min(nk);
n=nma+sum((nbkm-nkm))+nu;

vz=1;
if size(z)==size(zv)
  if norm(z-zv)<eps, vz=0;end
end

% *** Set up default values **
maxsdef=idmsize(Ncap,n);
maxsize=maxsdef;


% *** construct regression matrix ***
nmax=max(max([na+ones(nm,1) nb+nk]'))-1;

M=10000;  %M=floor(maxsize/n); (For avoiding OUT OF MEMORY!)

R=zeros(n);
F=zeros(n,1);
if vz,Rv=zeros(n);Fv=zeros(n,1);end

for k=nmax:M:max(Ncap,Ncapv)  %k=1:25000:4 yields 1
   if min(Ncap,k+M)<k+1, ntz=0; else ntz=1; end
   if min(Ncapv,k+M)<k+1, ntzv=0; else ntzv=1; end
   if ntz,jj=(k+1:min(Ncap,k+M));phi=zeros(length(jj),n);end
   
      if vz & ntzv,jjv=(k+1:min(Ncapv,k+M));phiv=zeros(length(jjv),n);end

      for kl=1:nma,
         if ntz,phi(:,kl)=-z(jj-kl,1);end
         if vz & ntzv ,phiv(:,kl)=-zv(jjv-kl,1);end
      end
      ss=nma;
      
      for ku=1:nu
         for kl=nkm(ku):nbkm(ku),
             if ntz,phi(:,ss+kl+1-nkm(ku))=z(jj-kl,ku+1);end
             if vz & ntzv,phiv(:,ss+kl+1-nkm(ku))=zv(jjv-kl,ku+1);end
         end
         ss=ss+nbkm(ku)-nkm(ku)+1;
      end
      if ntz,R=R+phi'*phi; F=F+phi'*z(jj,1);end
      if vz & ntzv,Rv=Rv+phiv'*phiv; Fv=Fv+phiv'*zv(jjv,1);end
end

jj=0;
for j=1:nm
   estparno=na(j)+sum(nb(j,:));
   if estparno>0
     jj=jj+1;
     s=[1:na(j)];
     rs=nma;
     for ku=1:nu
         s=[s,rs+nk(j,ku)-nkm(ku)+1:rs+nb(j,ku)+nk(j,ku)-nkm(ku)];
         rs=rs+nbkm(ku)-nkm(ku)+1;
     end
     RR=R(s,s);  %RR=X'*X
     FF=F(s);    %FF=X'*D
     if vz,RRv=Rv(s,s);FFv=Fv(s);end
     TH=(RR\FF);
   end % if estparno>0
end;

PHI=phi; 
PHIV=[];  if vz==1, PHIV=phiv; end
