function [order,fit,selected_poles,Energy] = model_reduction(ze,zv,true_delay,real_part,imaginary_part,no_iteration)

%MODEL_REDUCTION searches the ordering of the optimum dominating poles based on the system
%   energy criterion in the sense that the first sections contribute most to the overall impulse
%   response ofthe original system in a quadratic sense.      
%
%   ZE: the output-input data ZE=[ye ue] for estimation process,
%       with ye and ue as column vectors.
%
%   ZV: the output-input data ZV=[yv uv] for validation process,
%       with yv and uv as column vectors.
%
%   TRUE_DELAY: the true delays of the system 
%
%   REAL_PART: real_part = [started_point:step_size:ended_point] 
%              where abs(started_point) < 1 and abs(ended_point) < 1
%
%   IMAGINARY_PART: imaginary_part = [started_point:step_size:ended_point] 
%                   where abs(started_point) < 1 and abs(ended_point) < 1
%
%   NO_ITERATION: The maximum number of iteration used in this function to stop the process.  
%
%   ORDER: The minimum model order obtained from this function, based on the system energy criterion. 
%
%   FIT: The corresponding fit of error at obtained order and selected poles 
% 
%   SELECTED_POLES: The ordering of the chosen poles.
%
%   ENERGY: The energy calculated from the squared absolute value of coefficient weights.    
%      

%   Piya Kovintavewat
%   Revision:1       Date: 1998/10/22 

if nargin<6, no_iteration=50;  end;
if nargin<5, imaginary_part=0; end;
NK=true_delay-1; if NK<0, NK=0; end;
vz=1; if size(ze)==size(zv), if norm(ze-zv)<eps, vz=0; end;  end
maxlength = length(ze(:,1));  
if length(real_part)>1, step_size_r=real_part(2)-real_part(1); else, step_size_r=0; end;
if length(imaginary_part)>1, step_size_c=imaginary_part(2)-imaginary_part(1); else, step_size_c=0; end; 
selected_poles=[];  Energy=[];  sel_NB=0;             

for number_loop=1:no_iteration

    nmax=max([1, sel_NB+1+NK+1]);  ii=nmax:maxlength;  

%---Compute the energy of a first order section (only real poles)  
    temp_Energy=[];  temp_total_energy=[]; 
    for j=1:length(real_part)
        choice_pole = real_part(j);
        poles=[];  poles=[selected_poles, choice_pole]; 
        bPHI=[];   bPHIV=[];  [bPHI,bPHIV] = phi_gobfilt_arx(ze,ze,poles,0); 
        THETA= bPHI(ii-NK,:)\ze(ii,1);
        temp_Energy(j)=abs(THETA(sel_NB+1))^2;
        temp_total_energy(j)=sum(THETA.^2);  
    end
    [Ymax,imax]   = max(temp_Energy);   Emax=Ymax;
    total_energy1 = temp_total_energy(imax);
    r_pole     = real_part(imax);
    
%---Compute the energy of two additional first order sections for either complex-conjugated
%   pole pair or 2 identical real-valued poles. 
    temp_Energy=[];  temp_total_energy=[]; 
    nmax=max([1, sel_NB+2+NK+1]);  ii=nmax:maxlength;  
    for i=1:length(real_part)
        for j=1:length(imaginary_part)
            pole = real_part(i)+imaginary_part(j);
            if abs(pole) < 0.99
               if abs(imag(pole))==0, pole=ones(1,2)*pole; end;  
               poles = [];  poles = [selected_poles, pole];  
               bPHI  = [];  bPHIV=[];  [bPHI,bPHIV] = phi_gobfilt_arx(ze,ze,poles,0); 
               THETA = bPHI(ii-NK,:)\ze(ii,1);
               temp_Energy(i,j)=sum(abs(THETA(sel_NB+1:sel_NB+2)).^2);
               temp_total_energy(i,j)=sum(THETA.^2);  
            else
               temp_Energy(i,j)=0;  temp_total_energy(i,j)=0;
            end;
        end;
    end; 
    [yimax,irow]  = max(temp_Energy);           
    [Ymax,imax]   = max(max(temp_Energy));      
    total_energy2 = temp_total_energy(irow(imax),imax);  
    c_pole        = (real_part(1)+(irow(imax)-1)*step_size_r)+(imaginary_part(1)+(imax-1)*step_size_c);
    if abs(imag(c_pole))==0, c_pole=ones(1,2)*c_pole; end; 

%---Compare the increasing energy of real pole and that of complex pole
    if Ymax>Emax
       if isempty(Energy), Energy=total_energy2; else, Energy=[Energy,total_energy2]; end; 
       selected_poles = [selected_poles, c_pole];
       sel_NB=sel_NB+2;
    else
       if isempty(Energy), Energy=total_energy1; else, Energy=[Energy,total_energy1]; end;
       selected_poles = [selected_poles, r_pole];
       sel_NB=sel_NB+1; 
    end;

%---Check an ADDITIONAL ENERGY
    if length(Energy)>1
       Threshold = (Energy(length(Energy))-Energy(length(Energy)-1));
    end
    if (length(Energy)>1) & (Threshold < 0.001)  %---Less than 0.1% of the previous energy   
       selected_poles=selected_poles(1:length(selected_poles)-1);
       Energy=Energy(1:length(Energy)-1);
       break; 
    end; 
end;

real_p    =  selected_poles(find(abs(imag(selected_poles))==0));
complex_p =  selected_poles(find(abs(imag(selected_poles))>0));
order     =  length(real_p) + 2*length(complex_p);
poles     =  selected_poles;    

nmax = max([1, order+NK+1]);  ii=nmax:maxlength;  
bPHI = []; bPHIV=[];          [bPHI,bPHIV] = phi_gobfilt_arx(ze,zv,poles,0);  
THETA    = bPHI(ii-NK,:)\ze(ii,1);
if vz==0,  ysim=bPHI(ii-NK,:)*THETA; else, ysim=bPHIV(ii-NK,:)*THETA; end;
fit = norm(ysim-zv(ii,1))/sqrt(length(ysim))

