function [mPHI, mPHIV, GAMMA] = orthobase(ze, zv, poles, NAflag, w, T, NK)

%ORTHOBASE computes the regression matrices used in estimation/validation process and the matrix used for calculating
%	estimated model frequency response for orthonormal model.
%
%   [ mPHI, mPHIV, GAMMA ] = orthobase( ze, zv, poles, NAflag, w, T, NK)
% 
%   mPHI:  returned as the regression matrix used for estimation process.
% 
%   mPHIV: returned as the regression matrix used for validation process.
%   GAMMA:  Matrix such that the estimated model frequency response = GAMMA*THETA;
%
%   ZE: the output-input data, ZE=[ye ue] with ye and ue as column vectors, used for estimation process.
%
%   ZV: the output-input data, ZV=[yv uv] having the same size as ZE, used for validation process.
% 
%   POLES: is a matrix containing the set of chosen poles in a row. 
%
%   NAflag: The index used for indicating the type of model structures, i.e. NAflag=1 corresponds to ARX structure, while
%	 NAflag=0 corresponds to FIR structure.
%
%   W: Vector of frequencies to evaluate freq. response at NOT in normalized, but in real freq.
%
%   T: Normalized sampling time interval. 
%
%   NK: The true delay of the system.

%   Modified from orthobase.m which was written by Brett Ninness (First written 9/8/94).
%   Piya Kovintavewat
%   Revision:1       Date: 1998/08/08       

if nargin<7, NK = 0;    end;      
if nargin<6, T    = 1;    end;
if nargin<5, w    = [ ];  end;
if nargin<4, Naflag=0; end;   Aflag = NAflag;
NK=NK-1; if NK<0, NK=0; end;
vz=1; if size(ze)==size(zv), if norm(ze-zv)<eps, vz=0; end; end

mPHI = [ ];    mPHIV = [ ];  GAMMA = [ ];   
ww     = exp(j*w*T);   
ww_NK=exp(-j*w*T*NK);

%-------------------------Initialize the construction
num  = sqrt(1-abs(poles(1))^2);  
den   = [1,-poles(1)];

if abs(imag(poles(1))) > 0 
   %------- If a pole is complex we have to include its conjugate
   den   = conv(den,[1,-poles(1)']);             
   alpha = 2*real(poles(1))/(1+abs(poles(1))^2);

   if Aflag==0  %--FIR structure
      phi     = dlsim(num*sqrt((1-alpha^2)*(1+abs(poles(1))^2)), den, ze(:,2));           mPHI     = [mPHI, phi(:)];  
      if vz,  phiv  = dlsim(num*sqrt((1-alpha^2)*(1+abs(poles(1))^2)), den, zv(:,2));  mPHIV  = [mPHIV, phiv(:)];   end;
   else  %--ARX structure
      phi     = dlsim(num*sqrt((1-alpha^2)*(1+abs(poles(1))^2)), den, -ze(:,1));           mPHI     = [mPHI, phi(:)];  
      if vz,  phiv  = dlsim(num*sqrt((1-alpha^2)*(1+abs(poles(1))^2)), den, -zv(:,1));  mPHIV  = [mPHIV,phiv(:)];  end;
   end;
   gamma = polyval(num*sqrt((1-alpha^2)*(1+abs(poles(1))^2)), ww)./polyval(den,ww);  GAMMA = [GAMMA, gamma(:)];
   
   if Aflag==0  %--FIR structure
      phi     = dlsim(num*sqrt(1+abs(poles(1))^2)*[1,-alpha], den, ze(:,2));           mPHI    = [mPHI, phi(:)];
      if vz,  phiv  = dlsim(num*sqrt(1+abs(poles(1))^2)*[1,-alpha], den, zv(:,2));  mPHIV = [mPHIV,phiv(:)];    end;
   else   %--ARX structure
      phi     = dlsim(num*sqrt(1+abs(poles(1))^2)*[1,-alpha],den,-ze(:,1));             mPHI     = [mPHI, phi(:)];
      if vz,  phiv  = dlsim(num*sqrt(1+abs(poles(1))^2)*[1,-alpha],den,-zv(:,1));    mPHIV  = [mPHIV,phiv(:)];  end;
   end;
   gamma = polyval(num*sqrt(1+abs(poles(1))^2)*[1,-alpha], ww)./polyval(den, ww);   GAMMA = [GAMMA, gamma(:)];

   %-------  Now put complex-conjugated pole automatically for the next iteration.
   num = num*[-poles(1)',1];    

else  
   if Aflag==0  %--FIR structure
      phi  = dlsim(num, den, ze(:,2));             mPHI   = [mPHI, phi(:)];
      if vz, phiv = dlsim(num, den, zv(:,2));    mPHIV = [mPHIV, phiv(:)]; end;
   else  %--ARX structure
      phi    = dlsim(num, den, -ze(:,1));           mPHI    = [mPHI, phi(:)];
      if vz, phiv = dlsim(num, den, -zv(:,1));   mPHIV = [mPHIV, phiv(:)];   end;
   end;
   gamma = (polyval(num, ww)./polyval(den, ww)).*ww_NK;   GAMMA = [GAMMA, gamma(:)];
end;

%------------------------- Now iterate through for as many poles as specified.
for k = 2:length(poles)
    if abs(imag(poles(k))) > 0  
       %-------  First test to see if the pole is complex or not.
       num   = real(sqrt((1-abs(poles(k))^2)/(1-abs(poles(k-1))^2))*conv(num, [-poles(k-1),1]));
       den    = conv(den,[1,-poles(k)]);
       %------- If a pole is complex we have to include its conjugate automatically.
       den    = real(conv(den, [1,-poles(k)'])); 
       alpha = 2*real(poles(k))/(1+abs(poles(k))^2);

       if Aflag==0  %--FIR structure
          phi     = dlsim(num*sqrt((1-alpha^2)*(1+abs(poles(k))^2)), den, ze(:,2));             mPHI    = [mPHI, phi(:)];
          if vz,  phiv  = dlsim(num*sqrt((1-alpha^2)*(1+abs(poles(k))^2)), den, zv(:,2));    mPHIV = [mPHIV, phiv(:)];  end;
       else  %--ARX structure
          phi     = dlsim(num*sqrt((1-alpha^2)*(1+abs(poles(k))^2)),den,-ze(:,1));               mPHI   = [mPHI, phi(:)];
          if vz,  phiv  = dlsim(num*sqrt((1-alpha^2)*(1+abs(poles(k))^2)), den, -zv(:,1));   mPHIV = [mPHIV, phiv(:)];  end;
       end;
       gamma = polyval(num*sqrt((1-alpha^2)*(1+abs(poles(k))^2)),ww)./polyval(den,ww); GAMMA=[GAMMA, gamma(:)];
 
       if Aflag==0  %--FIR structure
          phi     = dlsim(sqrt(1+abs(poles(k))^2)*conv(num,[1,-alpha]), den, ze(:,2));            mPHI    = [mPHI, phi(:)];
          if vz,  phiv  = dlsim(sqrt(1+abs(poles(k))^2)*conv(num,[1,-alpha]), den, zv(:,2));   mPHIV = [mPHIV, phiv(:)]; end;
       else   %--ARX structure
          phi     = dlsim(sqrt(1+abs(poles(k))^2)*conv(num,[1,-alpha]), den, -ze(:,1));           mPHI     = [mPHI, phi(:)];
          if vz,  phiv  = dlsim(sqrt(1+abs(poles(k))^2)*conv(num,[1,-alpha]), den, -zv(:,1));  mPHIV  = [mPHIV, phiv(:)]; end;
       end
       gamma  = polyval(sqrt(1+abs(poles(k))^2)*conv(num, [1,-alpha]), ww)./polyval(den, ww); 
       GAMMA = [GAMMA, gamma(:)];

       %------- Now put complex-conjugated pole automatically for the next iteration.
       num = conv(num,[-poles(k), 1]);     
    
    else  
       num = real(sqrt((1-abs(poles(k))^2)/(1-abs(poles(k-1))^2))*conv(num,[-poles(k-1)', 1]));
       den  = conv(den,[1, -poles(k)]);
 
       if Aflag==0  %--FIR structure
          phi     = dlsim(num,den,ze(:,2));            mPHI    = [mPHI, phi(:)];
          if vz,  phiv=dlsim(num,den,zv(:,2));      mPHIV = [mPHIV, phiv(:)];   end;
       else   %--ARX structure
          phi     = dlsim(num,den,-ze(:,1));           mPHI    = [mPHI, phi(:)];
          if vz,  phiv = dlsim(num,den,-zv(:,1));   mPHIV = [mPHIV, phiv(:)];   end;
       end;
       gamma = (polyval(num, ww)./polyval(den, ww)).*ww_NK;    GAMMA = [GAMMA, gamma(:)];
    end;
end
