function  [mPHI,mPHIV] = phi_gobfilt_arx(ze,zv,poles,NAflag)

%PHI_GOBFILT_ARX computes the regressor vector in order to use it to find the estimated parameters
%
%   [mPHI,mPHIV] = phi_gobfilt_arx(ze,zv,poles,NAflag)
% 
%   mPHI:  returned as the regressor matrix used in estimation process  
% 
%   mPHIV: returned as the regressor matrix used in validation process  
%
%   ZE: the output-input data ZE=[ye ue] for estimation process,
%       with ye and ue as column vectors.
%
%   ZV: the output-input data ZV=[yv uv] having the same size as ZE for validation process,
%       with yv and uv as column vectors.
% 
%   POLES: It is a matrix containing the whole sequences of chosen pole. A number of elements
%       in this matrix depends on the type of model. That is, given n-th model order, a number
%       of elements is equal to n for LAGUERRE model, while it is equal to n/2 for KAUTZ model.
%
%   NAflag: The index uses for indicating the types of model structure, i.e. 
%       NAflag=1 corresponds to ARX structure, while NAflag=0 corresponds to FIR structure.
      
%   Piya Kovintavewat
%   Revision:1       Date: 1998/10/15       

if nargin<4, NAflag=0; end;
vz=1; if size(ze)==size(zv), if norm(ze-zv)<eps, vz=0; end; end;
ue_ap = ze(:,2);     ye_ap = ze(:,1);
uv_ap = zv(:,2);     yv_ap = zv(:,1); 
mPHI  = [ ];         mPHIV = [ ];

%---- uap = u filtered through the all pass filter up to pole k-1
for k = 1:length(poles)

    if ( abs( imag( poles(k) ) ) > 0 )  %---KAUTZ model (Complex pole)
       denk = [1,-poles(k)];
       %---If a pole is complex we have to include its conjugate autmomatically
       denk = real(conv(denk,[1,-poles(k)']));
       alpha = 2*real(poles(k))/(1+abs(poles(k))^2);
       %---Kautz Numerator
       numk1 = sqrt((1-alpha^2)*(1+abs(poles(k))^2)*(1-abs(poles(k))^2))*[0 0 1];
       numk2 = sqrt((1+abs(poles(k))^2)*(1-abs(poles(k))^2))*[0 1 -alpha];
       if NAflag==0   %---FIR structure
          phi1  = filter(numk1,denk,ue_ap);      phi2  = filter(numk2,denk,ue_ap);  
          mPHI  = [mPHI,phi1(:),phi2(:)];        ue_ap = filter([denk(3:-1:1)'],denk,ue_ap);
          if vz
             phiv1 = filter(numk1,denk,uv_ap);   phiv2 =  filter(numk2,denk,uv_ap);
             mPHIV = [mPHIV,phiv1(:),phiv2(:)];  uv_ap =  filter([denk(3:-1:1)'],denk,uv_ap);
          end;
       else           %---ARX structure
          phi1  = filter(numk1,denk,-ye_ap);     phi2  = filter(numk2,denk,-ye_ap);  
          mPHI  = [mPHI,phi1(:),phi2(:)];        ye_ap = filter([denk(3:-1:1)'],denk,ye_ap);
          if vz
             phiv1 = filter(numk1,denk,-yv_ap);  phiv2 =  filter(numk2,denk,-yv_ap);
             mPHIV = [mPHIV,phiv1(:),phiv2(:)];  yv_ap =  filter([denk(3:-1:1)'],denk,yv_ap);
          end;
       end; 
    else  %---LAGUERRE model (Real pole)
       numk = [0 sqrt(1-abs(poles(k))^2)];
       denk = [1,-poles(k)];
       if NAflag==0   %---FIR structure
          phi  = filter(numk,denk,ue_ap);
          mPHI = [mPHI,phi(:)];                  ue_ap=filter([denk(2:-1:1)'],denk,ue_ap);
          if vz
             phiv  = filter(numk,denk,uv_ap);
             mPHIV = [mPHIV,phiv(:)];            uv_ap=filter([denk(2:-1:1)'],denk,uv_ap);
          end;  
       else           %---ARX structure
          phi  = filter(numk,denk,-ye_ap);
          mPHI = [mPHI,phi(:)];                  ye_ap=filter([denk(2:-1:1)'],denk,ye_ap);
          if vz
             phiv  = filter(numk,denk,-yv_ap);
             mPHIV = [mPHIV,phiv(:)];            yv_ap=filter([denk(2:-1:1)'],denk,yv_ap);
          end;  
       end;
   end;
end;
